//================================================================================
//
//  This code is the intellectual property of Jerry Jongerius (duckware.com)
//
//================================================================================

//
//  Dedicated download+upload speedtest program (no httpd)
//

import java.net.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class speedtest implements Runnable {
    private final static int T_MAGIC=0xEABBCCD4;               // magic number
    private final static int T_DOWNLOAD=1;                     // type=download
    private final static int T_UPLOAD=2;                       // type=upload
    private final static int TEST_TIME_MS=11000+700;           // extra ms so that all ten buckets fill (both ways)
    private final static int FLUSH_TIME_MS=1000;               // flush this much first
    private final static byte g_buffer[]=makeBuffer(20000000); // maximum write buffer

    //----------------------------------------------------------------------
    private static String addCommas( long l ) {
        return NumberFormat.getNumberInstance(Locale.US).format(l);
        }
    //----------------------------------------------------------------------
    private static String tobps( long bps ) {
        return addCommas(bps)+" bps";
        }
    //----------------------------------------------------------------------
    private static byte[] makeBuffer( int nLen ) {
        byte buffer[] = new byte[nLen];
        for (int loop=0; loop<buffer.length; ++loop) {
            buffer[loop] = (byte)(Math.random()*255);
            }
        return buffer;
        }
    //----------------------------------------------------------------------
    public static void main( String[] args ) throws IOException {
        int nBuffering = 5*1024*1024;   // default buffering
        int nPort = 33333;              // default port

        if (args.length>0) {
            String host=null;

            // process args
            for (int loop=0; loop<args.length; ++loop) {
                String arg = args[loop];
                if (arg.startsWith("-port:")) {
                    nPort = Integer.parseInt(arg.substring(6));
                    }
                else if (arg.startsWith("-buffer:")) {
                    int V = Integer.parseInt(arg.substring(8));
                    nBuffering = Math.min(g_buffer.length,Math.max(1000,V));
                    }
                else {
                    host = arg;
                    }
                }

            // run a SERVER
            if ("server".equals(host)) {
                System.out.println( "Running SpeedTest SERVER on "+InetAddress.getLocalHost()+" port "+nPort+"." );
                System.out.println( "Waiting for clients to connect..." );
                ServerSocket ss = new ServerSocket(nPort);
                new Exiter();
                while (true) {
                    new speedtest(ss.accept());
                    }
                }
            // run a CLIENT
            else if (null!=host) {
                runClientDownload( host, nPort, nBuffering );
                runClientUpload( host, nPort, nBuffering );
                }
            // error
            else {
                }
            }

        // program help (no command line args)
        else {
            System.out.println( "Download/Upload throughput speed test v3.0a - by Jerry Jongerius (duckware.com)" );
            System.out.println( "" );
            System.out.println( "Usage: java -jar speedtest.jar [-port:#] [-buffer:#] <host>" );
            System.out.println( "" );
            System.out.println( "  Options:" );
            System.out.println( "    -port:#    tcp/ip port number (default: "+nPort+")" );
            System.out.println( "    -buffer:#  socket buffer size in bytes (default: "+nBuffering+")" );
            System.out.println( "" );
            System.out.println( "  To run a 'server', set <host> to \"server\"" );
            System.out.println( "  To run a 'client', set <host> to client IP address" );
            }
        }
    //----------------------------------------------------------------------
    private static String pp( SocketAddress sa ) {
        try { InetSocketAddress a = (InetSocketAddress)sa; return a.getAddress().getHostAddress()+" port "+a.getPort(); } catch (Exception e) {}
        return ""+sa;
        }
    //----------------------------------------------------------------------
    private static void runClientDownload( String host, int nPort, int nBuffering ) throws IOException {
        Socket s = new Socket();
        s.setReceiveBufferSize( nBuffering );
        s.connect( new InetSocketAddress(host, nPort) );
        System.out.println( "Downloading from "+pp(s.getRemoteSocketAddress())+" (buffer="+s.getReceiveBufferSize()+")..." );
        DataInputStream in = new DataInputStream(s.getInputStream());
        DataOutputStream out = new DataOutputStream(s.getOutputStream());
        out.writeInt( T_MAGIC );
        out.writeInt( nBuffering );
        out.writeInt( T_DOWNLOAD );
        doRead( in, System.out, nBuffering );
        s.close();
        }
    //----------------------------------------------------------------------
    private static void runClientUpload( String host, int nPort, int nBuffering ) throws IOException {
        Socket s = new Socket();
        s.setSendBufferSize( nBuffering );
        s.connect( new InetSocketAddress(host, nPort) );
        System.out.println( "Uploading to "+pp(s.getRemoteSocketAddress())+" (buffer="+s.getSendBufferSize()+")..." );
        DataInputStream in = new DataInputStream(s.getInputStream());
        DataOutputStream out = new DataOutputStream(s.getOutputStream());
        out.writeInt( T_MAGIC );
        out.writeInt( nBuffering );
        out.writeInt( T_UPLOAD );
        doWrite( in, out, true, nBuffering );
        s.close();
        }

    //====================================================================== 
    //====================================================================== 
    //====================================================================== 

    // very intentionally want the SAME read/write code running on both client and server

    //----------------------------------------------------------------------
    private static void doRead( InputStream in, PrintStream out, int nBuffering ) throws IOException {
        Printer p = new Printer(out);

        try {
            byte by[] = new byte[Math.min(nBuffering,g_buffer.length)];   // read buffer size
            // flush
            long t1 = G.tick();
            while (((G.tick()-t1)<FLUSH_TIME_MS) && (in.read(by)>0)) {
                }
            // download
            t1 = G.tick();
            long lSum=0;
            int nRead;
            while ((nRead=in.read(by))>0) {
                lSum += nRead;
                long t2 = G.tick();
                if (t2-t1>=1000) {
                    p.print( "  "+addCommas(lSum)+" bytes in "+(t2-t1)+" ms = "+tobps(lSum*8L*1000/(t2-t1))+"\r\n" );
                    lSum = 0;
                    t1 = t2;
                    }
                }
            }
        finally {
            p.end();
            }
        }
    //----------------------------------------------------------------------
    private static void doWrite( InputStream in, OutputStream out, boolean bCheckInput, int nBuffering ) throws IOException {
        Printer p = new Printer(System.out);
        try {
            byte by[] = new byte[10000];
            long t1 = G.tick();
            long tx = t1;
            nBuffering = Math.min(nBuffering,g_buffer.length);
            while ((G.tick()-t1)<TEST_TIME_MS) {
                out.write( g_buffer, 0, nBuffering );
                long tNow = G.tick();
                if (bCheckInput && (tNow-tx>=500)) {
                    int nRead;
                    tx = tNow;
                    while ((in.available()>0) && ((nRead=in.read(by))>0)) {
                        p.print( new String(by,0,nRead) );
                        }
                    }
                }
            }
        finally {
            p.end();
            }
        }

    //====================================================================== 
    //====================================================================== 
    //====================================================================== 

    // only for server mode -- handle incoming download/upload requests

    private Socket m_s;

    //----------------------------------------------------------------------
    private speedtest( Socket s ) {
        m_s = s;
        (new Thread(this)).start();
        }
    //----------------------------------------------------------------------
    public void run() {
        try {
            DataInputStream dis = new DataInputStream(m_s.getInputStream());
            PrintStream out = new PrintStream(m_s.getOutputStream());
            if (T_MAGIC==dis.readInt()) {
                int nBuffering = dis.readInt();
                int nAction = dis.readInt();
                if (T_DOWNLOAD==nAction) {
                    m_s.setSendBufferSize( nBuffering );
                    System.out.println( "Uploading to "+pp(m_s.getRemoteSocketAddress())+" (buffer="+m_s.getSendBufferSize()+")..." );
                    doWrite( dis, out, false, nBuffering );
                    }
                else if (T_UPLOAD==nAction) {
                    m_s.setReceiveBufferSize( nBuffering );
                    System.out.println( "Downloading from "+pp(m_s.getRemoteSocketAddress())+" (buffer="+m_s.getReceiveBufferSize()+")..." );
                    doRead( dis, out, nBuffering );
                    }
                }
            m_s.close();
            }
        catch (IOException e) {
            e.printStackTrace();
            }
        }
    }